# Generated by Django 4.2.18 on 2025-03-23 19:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('account', '0022_alter_memberdevice_modified'),
    ]

    operations = [
        migrations.AddField(
            model_name='authsession',
            name='method',
            field=models.CharField(blank=True, db_index=True, default=None, max_length=127, null=True),
        ),
    ]
