# Generated by Django 4.2.11 on 2025-01-03 19:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('incident', '0015_rule_title_template_alter_incident_state'),
    ]

    operations = [
        migrations.AddField(
            model_name='rule',
            name='notify_template',
            field=models.TextField(default=None, null=True),
        ),
    ]
